﻿using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

namespace Bonfire
{
    public class PlayerMoveRigidbody : MonoBehaviour
    {
        [SerializeField] private Text debugText = null;
        [SerializeField] private Transform lookTarget = null;

        // 重力値
        const float GravityPower = 9.8f;

        // 移動速度
        float walkSpeed = 4f;
        float runSpeedUpRatio = 2.0f;
        float sprintSpeedUpRatio = 2.0f;

        // 回転速度
        [SerializeField] private float rotationSpeed = 720.0f;

        // ジャンプの強さ
        float jumpPower = 10f;
        // あまり速いと地面を突き抜けそうな気がするので、最高落下速度を決めておく。
        float maxFallSpeed = -30f;

        // 現在の速度 (実際に使用するのは y 方向のみ)
        Vector3 currentVelocity = Vector3.zero;

        // 着地しているかどうか
        bool isGrounded = false;

        // 着地判定に飛ばす Ray の距離
        [SerializeField] float isGroundedTestRayDistance = 0.15f;


        Camera mainCamera = null;
        Animator animator = null;
        PlayerCtrl playerCtrl = null;
        PlayerStatus status = null;
        CharacterController characterController = null;

        // RIGIDBODY
        Rigidbody rbody = null;
        // 移動速度
        Vector3 velocity = Vector3.zero;
        float moveSpeed = 0f;
        bool isJumpInputted = false;

        void Start()
        {
            mainCamera = FindObjectOfType<Camera>();
            animator = GetComponent<Animator>();
            playerCtrl = GetComponent<PlayerCtrl>();
            status = GetComponent<PlayerStatus>();
            characterController = GetComponent<CharacterController>();

            // RIGIDBODY
            rbody = GetComponent<Rigidbody>();
        }

        void Update()
        {
            float speedByStick = 0f;
            float runSpeedUp = 1.0f;
            float sprintSpeedUp = 1.0f;

            // 独自の着地判定
            Ray ray = new Ray(transform.position + 0.18f * transform.up, -transform.up);
            isGrounded = Physics.SphereCast(ray, 0.13f, isGroundedTestRayDistance);
            status.isGrounded = isGrounded;

            if (playerCtrl.IsInputStickL())
            {
                // プレイヤーの向き
                Vector3 playerDirection = playerCtrl.GetPlayerDirection();

                // カメラの向きに、スティックの向きを掛け合わせ、キャラクターをスティックの方向に向かせる
                Vector3 cameraDirection = lookTarget.position - mainCamera.transform.position;
                cameraDirection.y = 0;
                cameraDirection = Vector3.Normalize(cameraDirection);
                Quaternion characterTargetRotation = Quaternion.LookRotation(cameraDirection) * Quaternion.LookRotation(playerDirection);
                transform.rotation = Quaternion.RotateTowards(transform.rotation, characterTargetRotation, rotationSpeed * Time.deltaTime);

                // x ボタンでの加速
                if (playerCtrl.isRunning)
                {
                    runSpeedUp = runSpeedUpRatio;
                }
                else
                {
                    // x ボタンを離すと Sprint も中止
                    playerCtrl.isSprint = false;
                }
                // L3 ボタンでのsprint加速
                if (playerCtrl.isSprint)
                {
                    sprintSpeedUp = sprintSpeedUpRatio;
                }

                // スティックの傾きによる速さの変化
                speedByStick = playerCtrl.GetMagunitudeStickL();

                moveSpeed = speedByStick * walkSpeed * runSpeedUp * sprintSpeedUp;
                // RIGIDBODY
                // velocity = transform.forward.normalized * moveSpeed;
            }
            else
            {
                moveSpeed = 0f;
            }

            // アニメーションの速度設定
            animator.SetFloat("Speed", moveSpeed);
            animator.SetFloat("SpeedVertical", velocity.y);

        }

        void FixedUpdate()
        {
            currentVelocity = rbody.velocity;
            currentVelocity.y = Mathf.Max(currentVelocity.y, maxFallSpeed);

            velocity = transform.forward * moveSpeed;
            velocity.y = currentVelocity.y;

            // 1回だけジャンプが呼ばれるようにする
            if (isJumpInputted)
            {
                isJumpInputted = false;
                velocity.y = jumpPower;
            }

            // 着地判定
            if (status.isGrounded && velocity.y < 0.1f)
            {
                status.isJumping = false;
            }

            // --- MovePosition を使う場合 ---
            // 以下の Time.deltaTime は、Time.fixDeltaTime ではなく Time.deltaTime でいいらしい
            //（同じ値が返ってくるとのこと）
            // rbody.MovePosition(transform.position + transform.forward * moveSpeed * Time.deltaTime);
            // --- verocity を直接変更する場合 ---
            rbody.velocity = velocity;

            // Debug 
            if (debugText != null)
            {
                debugText.text =
                   String.Format("isGrounded: {0}\ncc.isGrounded: {1}\nSpeed: {2}\nvelocity.y: {3: 0; -0}\ncurrentVelocity: {4}\npos: {5}",
                                isGrounded,
                                characterController.isGrounded,
                                moveSpeed,
                                velocity.y,
                                currentVelocity,
                                transform.position.y
                               );
            }
        }

        public void StartJump()
        {
            isJumpInputted = true;
        }

        // Call by Unity
        private void OnDrawGizmosSelected()
        {
            // 着地判定の範囲を可視化する
            Gizmos.color = Color.blue;
            Gizmos.DrawWireSphere(transform.position + (0.18f - isGroundedTestRayDistance) * transform.up, 0.13f);
        }

    }

}